import os
import sys
from distutils.core import setup


setup(name='yoctopuce',
version='1.10.25748',
packages=['yoctopuce'],
description="Yoctopuce python API",
author="Yoctopuce",
author_email="support@yoctopuce.com",
url="http://www.yoctopuce.com",
include_package_data=True,
keywords = 'Yoctopuce USB sensors actuators API',
zip_safe=False,
 classifiers=[
'Development Status :: 5 - Production/Stable',
'Intended Audience :: Customer Service',
'Intended Audience :: Developers',
'Intended Audience :: Science/Research',
'Natural Language :: English',
'Operating System :: MacOS :: MacOS X',
'Operating System :: Microsoft :: Windows :: Windows XP',
'Operating System :: Microsoft :: Windows :: Windows Server 2008',
'Operating System :: Microsoft :: Windows :: Windows Vista',
'Operating System :: Microsoft :: Windows :: Windows 7',
'Operating System :: Unix',
'Programming Language :: Python :: 2.6',
'Programming Language :: Python :: 2.7',
'Programming Language :: Python :: 3',
'Programming Language :: Python :: 3.0',
'Programming Language :: Python :: 3.1',
'Programming Language :: Python :: 3.2',
'Programming Language :: Python :: 3.3',
]
)
